require 'shellwords'
require 'fileutils'
require 'open3'

class MermaidCLI
  def yarn_installed?
    yarn = `which yarn`
    puts "Yarn resolved to #{yarn}"
    if yarn.nil? || yarn == ""
      return false
    end

    true
  end

  def mermaid_installed?
    File.exist?(old_mermaid_path) && File.exist?(new_mermaid_path)
  end

  def ensure_latest_mermaid
    raise "Yarn not installed" unless yarn_installed?
    install_mermaid
    use_latest_mermaid_for_cli
    yield
  end

  def app_dir
    File.expand_path("../", __dir__)
  end

  def install_mermaid
    puts "mermaid not installed. Installing..."
    unless mermaid_installed?
      Dir.chdir app_dir do
        system("yarn install")
      end
    end
  end

  def old_mermaid_path
    File.expand_path("../node_modules/mermaid.cli/mermaid.min.js", __dir__)
  end

  def new_mermaid_path
    File.expand_path("../node_modules/mermaid/dist/mermaid.min.js", __dir__)
  end

  def use_latest_mermaid_for_cli
    puts "Upgrading mermaid for cli"
    FileUtils.cp(new_mermaid_path, old_mermaid_path)
  end

  def compile(src_path, dest_pdf_path)
    parts = [
      "yarn", "run", "mmdc",
      "-i", src_path,
      "-C", css_path,
      "-c", config_path,
      "-p", puppeteer_config_path,
      "-o", dest_pdf_path
    ].map { |part| Shellwords.escape(part) }

    cmd = parts.join(" ")

    Dir.chdir app_dir do
      ensure_latest_mermaid { system cmd }
    end
  end

  def css_path
    File.expand_path("../mermaid.css", __dir__)
  end

  def config_path
    File.expand_path("../mermaid.json", __dir__)
  end

  def puppeteer_config_path
    File.expand_path("../puppeteer.json", __dir__)
  end

end
